@@ -31,6 +31,7 @@
  *  with this program; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+#define _GNU_SOURCE
 #if HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -124,13 +125,6 @@
 #endif
 #include <pcap.h>
 
-#if defined(HAVE_SECCOMP_H) && defined(HAVE_LIBSECCOMP)
-#define USE_SECCOMP 1
-#include <seccomp.h>
-#else
-#define USE_SECCOMP 0
-#endif
-
 #include "arping.h"
 #include "cast.h"
 
@@ -194,6 +188,9 @@ static uint8_t dstmac[ETH_ALEN];
 static char* payload_suffix = NULL;
 static size_t payload_suffix_size = 4;
 
+/* If there were any libnet write failures, we return error. */
+static size_t libnet_write_failures = 0;
+
 uint32_t srcip;                   /* autodetected, override with -S/-b/-0 */
 uint8_t srcmac[ETH_ALEN];         /* autodetected, override with -s */
 
@@ -430,7 +427,7 @@ drop_fs_root()
         struct passwd *pw;
         errno = 0;
         if (!(pw = getpwnam(chroot_user))) {
-                if (verbose) {
+                if (verbose > 1) {
                         printf("arping: getpwnam(%s): %s\n",
                                chroot_user, strerror(errno));
                 }
@@ -620,73 +617,6 @@ drop_privileges(const char* drop_group)
 #endif
 }
 
-#if USE_SECCOMP
-static void seccomp_allow(scmp_filter_ctx ctx, const char* name)
-{
-        if (seccomp_rule_add(ctx, SCMP_ACT_ALLOW, seccomp_syscall_resolve_name(name), 0)) {
-                if (verbose) {
-                        fprintf(stderr, "arping: seccomp_rule_add_exact(%s): %s",
-                                name, strerror(errno));
-                }
-        }
-}
-
-static void drop_seccomp(int libnet_fd)
-{
-        //scmp_filter_ctx ctx = seccomp_init(SCMP_ACT_ERRNO(13));
-        scmp_filter_ctx ctx = seccomp_init(SCMP_ACT_KILL);
-        if (!ctx) {
-                perror("seccomp_init()");
-                exit(1);
-        }
-
-        //
-        // Whitelist.
-        //
-
-        // Write to stdout and stderr.
-        if (seccomp_rule_add(ctx, SCMP_ACT_ALLOW, SCMP_SYS(fstat), 1, SCMP_A0(SCMP_CMP_EQ, STDOUT_FILENO))) {
-                perror("seccomp_rule_add(fstat stdout)");
-                exit(1);
-        }
-        if (seccomp_rule_add(ctx, SCMP_ACT_ALLOW, SCMP_SYS(write), 1, SCMP_A0(SCMP_CMP_EQ, STDOUT_FILENO))) {
-                perror("seccomp_rule_add(write stdout)");
-                exit(1);
-        }
-        if (seccomp_rule_add(ctx, SCMP_ACT_ALLOW, SCMP_SYS(write), 1, SCMP_A0(SCMP_CMP_EQ, STDERR_FILENO))) {
-                perror("seccomp_rule_add(write stderr)");
-                exit(1);
-        }
-
-        // Libnet.
-        if (seccomp_rule_add(ctx, SCMP_ACT_ALLOW, SCMP_SYS(ioctl), 1, SCMP_A0(SCMP_CMP_EQ, cast_int_uint(libnet_fd, NULL)))) {
-                perror("seccomp_rule_add(ioctl libnet)");
-                exit(1);
-        }
-        if (seccomp_rule_add(ctx, SCMP_ACT_ALLOW, SCMP_SYS(sendto), 1, SCMP_A0(SCMP_CMP_EQ, cast_int_uint(libnet_fd, NULL)))) {
-                perror("seccomp_rule_add(sendto libnet)");
-                exit(1);
-        }
-
-        // Other.
-        seccomp_allow(ctx, "select");
-        seccomp_allow(ctx, "pselect6");
-        seccomp_allow(ctx, "newfstatat");
-        seccomp_allow(ctx, "exit_group");
-        seccomp_allow(ctx, "rt_sigreturn");
-
-        // Load.
-        if (seccomp_load(ctx)) {
-                perror("seccomp_load()");
-                exit(1);
-        }
-        seccomp_release(ctx);
-        if (verbose > 1) {
-                printf("arping: Successfully applied seccomp policy\n");
-        }
-}
-#endif
-
 /**
  * drop even more privileges, where possible.
  *
@@ -705,11 +635,9 @@ drop_more_privileges(int libnet_fd)
                 printf("arping: Successfully pledged\n");
         }
 #endif
-#if USE_SECCOMP
         if (use_seccomp) {
                 drop_seccomp(libnet_fd);
         }
-#endif
 }
 
 /**
@@ -1417,6 +1345,7 @@ pingmac_send(uint16_t id, uint16_t seq)
         if (-1 == libnet_write(libnet)) {
 		fprintf(stderr, "arping: libnet_write(): %s\n",
 			libnet_geterror(libnet));
+                libnet_write_failures++;
 		sigint(0);
 	}
         getclock(&lastpacketsent);
@@ -1489,9 +1418,13 @@ pingip_send()
                        (long)lastpacketsent.tv_sec,
                        (long)lastpacketsent.tv_nsec);
 	}
+        if (send_reply) {
+                printf("Sending ARP reply\n");
+        }
 	if (-1 == libnet_write(libnet)) {
 		fprintf(stderr, "arping: libnet_write(): %s\n",
 			libnet_geterror(libnet));
+                libnet_write_failures++;
 		sigint(0);
 	}
         getclock(&lastpacketsent);
@@ -1914,6 +1847,15 @@ ping_recv(pcap_t *pcap, uint32_t packetwait, pcap_handler func)
        int fd;
        unsigned int old_received;
 
+       if (send_reply) {
+               if (verbose > 3) {
+                       printf("arping: sending replies. Waiting %ld us…\n",
+                              (long)packetwait);
+               }
+               usleep(packetwait);
+               return;
+       }
+
        if (verbose > 3) {
                printf("arping: receiving packets...\n");
        }
@@ -2731,7 +2673,13 @@ arping_main(int argc, char **argv)
         if (finddup) {
                 return dupfound;
         }
-        return !numrecvd;
+        if (libnet_write_failures > 0) {
+                return EXIT_FAILURE;
+        }
+        if ((numrecvd > 0) || send_reply) {
+                return EXIT_SUCCESS;
+        }
+        return EXIT_FAILURE;
 }
 /* ---- Emacs Variables ----
  * Local Variables:
